<?php

/**
 * Query printer using D3.js library for creating a radar for technology foresight.
 */


class SRFRadar extends SMWAggregatablePrinter {

	/**
	 * Get a human readable label for this printer.
	 * @see SMWResultPrinter::getName()
	 * 
	 * @return string
	 */
	public function getName() {
		// Result printer name 
		return wfMessage( 'srf-printername-radar' )->text();
	}

	/**
	 * @see SMWResultPrinter::getResultText
	 *
	 * @param SMWQueryResult $result
	 * @param $outputMode
	 *
	 * @return string
	 */
	protected function getResultText( SMWQueryResult $result, $outputMode ) {
		
		// Data processing
		while ( $row = $result->getNext() ) {
			// Loop over the fields (properties)
			

			$label = array();


			foreach ( $row as /* SMWResultArray */ $field ) {
				
				$rowNumbers = array();
				
				// Property label
				$groupedBy = $field->getResultSubject()->getTitle()->getText(); 

				// Loop over all values for the property.
				while ( ( /* SMWDataValue */ $object = $field->getNextDataValue() ) !== false ) {

					if ( $object->getDataItem()->getDIType() == SMWDataItem::TYPE_NUMBER ) {
						$number =  $object->getNumber();

						if ( $label !== ''  ){
							$order = array($groupedBy, $number);
							$data[$label][] = $order;
						}
					}

					else {

						$label = $object->getWikiValue();
					}
				}
			}
		}

		// Check if the data processing returned any results otherwise just bailout
		if ( $data === array() ) {
			// Add an error message to return method
			return $result->addErrors( array( wfMessage( 'srf-no-results' )->inContentLanguage()->text() ) );
		
		} else {

			$options = array(
				'mode' => $outputMode
			);
			//var_dump($data);
			// Return formatted results
			return $this->getFormatOutput( $data );
		}
	}


	/**
	 * Prepare data for the output
	 *
	 * @param array $data
	 *
	 * @return string
	 */
	protected function getFormatOutput( array $data ) {
		

		// The generated ID is to distinguish similar instances of the same
		// printer that can appear within the same page
		static $statNr = 0;
		$radarID = 'srf-radar-' . ++$statNr;

		// Used to set that the output and being treated as HTML (opposed to plain wiki text)
		$this->isHTML = true;

		// Reorganize the raw data
		foreach ( $data as $name => $value ) {
				
				$dataObject[] = array( 'axis' => [$name] , 'value' => $value  );
		}

		// Prepare transfer objects
		$radardata = array (
			'data' => $dataObject,
			'parameters' => array (
			'middle'   => $this->params['middle'],
			'legend'  => $this->params['legend']
			)
		);

		// Ensure right conversion
		$width = strstr( $this->params['width'] ,"%") ? $this->params['width'] : $this->params['width'] . 'px';

		// Assign the ID to make a data instance readly available and distinguishable
		// from other content within the same page
		$requireHeadItem = array ( $radarID => FormatJson::encode( $radardata ) );
		SMWOutputs::requireHeadItem( $radarID, Skin::makeVariablesScript( $requireHeadItem ) );

		// Add resource definitions that has been registered with SRF_Resource.php
		// Resource definitions contain scripts, styles, messages etc.
		// SMWOutputs::requireResource( 'ext.srf.radar.namespace' );
		SMWOutputs::requireResource( "ext.srf.radar.namespace" );

		// Prepares an HTML element showing a rotating spinner indicating that something
		// will appear at this placeholder. The element will be visible as for as
		// long as jquery is not loaded and the JS plugin did not hide/removed the element.
		$processing = SRFUtils::htmlProcessingElement();

		// Add two elements a outer wrapper that is assigned a class which the JS plugin
		// can select and will fetch all instances of the same result printer and an innner
		// container which is set invisible (display=none) for as long as the JS plugin
		// holds the content hidden. It is normally the place where the "hard work"
		// is done hidden from the user until it is ready.
		// The JS plugin can prepare the output within this container without presenting
		// unfinished visual content, to avoid screen clutter and improve user experience.
		// Chart/graph placeholder
		$chart = Html::rawElement( 'div' , array(
			'id'    => $radarID,
			'class' => 'container',
			), null
		);

		//Create buttons for label rotation
		$button1 = Html::rawElement( 'input' , array(
			'type'    => 'button',
			'id' => 'rotateRight',
			'value' => 'Zeige Label rechts',
			), null
		);

		$button2 = Html::rawElement( 'input' , array(
			'type'    => 'button',
			'id' => 'reset',
			'value' => 'Reset',			
			), null
		);

		$button3 = Html::rawElement( 'input' , array(
			'type'    => 'button',
			'id' => 'rotateLeft',
			'value' => 'Zeige Label links',			
			), null
		);;
		
		// Processing placeholder
		$processing = SRFUtils::htmlProcessingElement( $this->isHTML );


		// D3 wrappper
		return Html::rawElement('div' , array(
			'class' => 'srf-radar',
			'style' => "width:{$width}; height:{$this->params['height']}px;"
			), $button1 . $button3 . $button2 . $processing . $chart
		);
	}

	/**
	 * @see SMWResultPrinter::getParamDefinitions
	 *
	 * @param $definitions array of IParamDefinition
	 *
	 * @return array of IParamDefinition|array
	 */
	public function getParamDefinitions( array $definitions ) {
		$params = parent::getParamDefinitions( $definitions );


		$params['height'] = array(
			'type' => 'integer',
			'message' => 'srf_paramdesc_chartheight',
			'default' => 400,
			'lowerbound' => 1,
		);

		$params['width'] = array(
			'message' => 'srf_paramdesc_chartwidth',
			'default' => 400,
		);

		$params['legend'] = array(
			'message' => 'srf_paramdesc_legend',
			'default' => 'Potential',
		);

		$params['middle'] = array(
			'message' => 'srf-paramdesc-middle',
			'default' => 'Relevanz',
		);

		return $params;
	}
}



